@@ -562,6 +562,14 @@ bool square_isdtrap(struct chunk *c, int y, int x) {
 	return sqinfo_has(c->squares[y][x].info, SQUARE_DTRAP);
 }
 
+/**
+ * True if cave square is inappropriate to place stairs
+ */
+bool square_isno_stairs(struct chunk *c, int y, int x) {
+	assert(square_in_bounds(c, y, x));
+	return sqinfo_has(c->squares[y][x].info, SQUARE_NO_STAIRS);
+}
+
 
 /**
  * SQUARE BEHAVIOR PREDICATES
@@ -853,6 +861,26 @@ bool square_isbelievedwall(struct chunk *c, int y, int x)
 	return !square_isprojectable(player->cave, y, x);
 }
 
+/**
+ * Checks if a square is in a cul-de-sac
+ */
+bool square_suits_stairs_well(struct chunk *c, int y, int x)
+{
+	if (square_isvault(c, y, x) || square_isno_stairs(c, y, x)) return false;
+	return (square_num_walls_adjacent(c, y, x) == 3) &&
+		(square_num_walls_diagonal(c, y, x) == 4) && square_isempty(c, y, x);
+}
+
+/**
+ * Checks if a square is in a corridor
+ */
+bool square_suits_stairs_ok(struct chunk *c, int y, int x)
+{
+	if (square_isvault(c, y, x) || square_isno_stairs(c, y, x)) return false;
+	return (square_num_walls_adjacent(c, y, x) == 2) &&
+		(square_num_walls_diagonal(c, y, x) == 4) && square_isempty(c, y, x);
+}
+
 
 
 /**
@@ -982,6 +1010,47 @@ void square_know_pile(struct chunk *c, int y, int x)
 }
 
 
+/**
+ * Return how many cardinal directions around (x, y) contain walls.
+ * \param c current chunk
+ * \param y co-ordinates
+ * \param x co-ordinates
+ * \return the number of walls
+ */
+int square_num_walls_adjacent(struct chunk *c, int y, int x)
+{
+    int k = 0;
+    assert(square_in_bounds(c, y, x));
+
+    if (feat_is_wall(c->squares[y + 1][x].feat)) k++;
+    if (feat_is_wall(c->squares[y - 1][x].feat)) k++;
+    if (feat_is_wall(c->squares[y][x + 1].feat)) k++;
+    if (feat_is_wall(c->squares[y][x - 1].feat)) k++;
+
+    return k;
+}
+
+/**
+ * Return how many diagonal directions around (x, y) contain walls.
+ * \param c current chunk
+ * \param y co-ordinates
+ * \param x co-ordinates
+ * \return the number of walls
+ */
+int square_num_walls_diagonal(struct chunk *c, int y, int x)
+{
+    int k = 0;
+    assert(square_in_bounds(c, y, x));
+
+    if (feat_is_wall(c->squares[y + 1][x + 1].feat)) k++;
+    if (feat_is_wall(c->squares[y - 1][x - 1].feat)) k++;
+    if (feat_is_wall(c->squares[y - 1][x + 1].feat)) k++;
+    if (feat_is_wall(c->squares[y + 1][x - 1].feat)) k++;
+
+    return k;
+}
+
+
 /**
  * Set the terrain type for a square.
  *
