@@ -885,7 +885,7 @@ void on_new_level(void)
 
 	/* Track maximum dungeon level */
 	if (player->max_depth < player->depth)
-		player->max_depth = player->depth;
+		player->max_depth = player->recall_depth = player->depth;
 
 	/* Flush messages */
 	event_signal(EVENT_MESSAGE_FLUSH);
