@@ -21,15 +21,25 @@
 
 #include "cave.h"
 
-u16b daycount;
-u32b seed_randart;
-u32b seed_flavor;
-s32b turn;
-bool character_generated;
-bool character_dungeon;
-bool character_saved;
-const byte extract_energy[200];
+struct level {
+	int depth;
+	char *name;
+	char *up;
+	char *down;
+	struct level *next;
+};
 
+extern u16b daycount;
+extern u32b seed_randart;
+extern u32b seed_flavor;
+extern s32b turn;
+extern bool character_generated;
+extern bool character_dungeon;
+extern const byte extract_energy[200];
+extern struct level *world;
+
+struct level *level_by_name(char *name);
+struct level *level_by_depth(int depth);
 bool is_daytime(void);
 int turn_energy(int speed);
 void play_ambient_sound(void);
