@@ -326,7 +326,7 @@ void place_secret_door(struct chunk *c, int y, int x);
 void place_closed_door(struct chunk *c, int y, int x);
 void place_random_door(struct chunk *c, int y, int x);
 void place_random_stairs(struct chunk *c, int y, int x);
-void alloc_stairs(struct chunk *c, int feat, int num, int walls);
+void alloc_stairs(struct chunk *c, int feat, int num);
 void vault_objects(struct chunk *c, int y, int x, int depth, int num);
 void vault_traps(struct chunk *c, int y, int x, int yd, int xd, int num);
 void vault_monsters(struct chunk *c, int y1, int x1, int depth, int num);
