@@ -10,6 +10,6 @@ MFLAG(NICE,		"Monster is still being nice")
 MFLAG(SHOW,		"Monster is recently memorized")
 MFLAG(MARK,		"Monster is currently memorized")
 MFLAG(VISIBLE,	"Monster is \"visible\"")
-MFLAG(UNAWARE,	"Player doesn't know this is a monster")
+MFLAG(CAMOUFLAGE,"Player doesn't know this is a monster")
 MFLAG(AWARE,	"Monster is aware of the player")
 MFLAG(HANDLED,	"Monster has been processed this turn")
