@@ -17,10 +17,7 @@ SQUARE(ROOM,		"part of a room")
 SQUARE(SEEN,		"seen flag")
 SQUARE(VIEW,		"view flag")
 SQUARE(WASSEEN,		"previously seen (during update)")
-SQUARE(DTRAP,		"trap detected square")
 SQUARE(FEEL,		"hidden points to trigger feelings")
-SQUARE(DEDGE,		"border of trap detected area")
-SQUARE(VERT,		"use an alternate visual for this square")
 SQUARE(TRAP,		"square containing a known trap")
 SQUARE(INVIS,		"square containing an unknown trap")
 SQUARE(WALL_INNER,	"inner wall generation flag")
@@ -31,3 +28,5 @@ SQUARE(NO_TELEPORT,	"player can't teleport from this square")
 SQUARE(NO_MAP,		"square can't be magically mapped")
 SQUARE(NO_ESP,		"telepathy doesn't work on this square")
 SQUARE(PROJECT,		"marked for projection processing")
+SQUARE(DTRAP,		"trap detected square")
+SQUARE(NO_STAIRS,	"square is not suitable for placing stairs")
