@@ -37,7 +37,8 @@ TF(DOOR_LOCKED,"Is a locked door")
 TF(MAGMA,      "Is a magma seam")
 TF(QUARTZ,     "Is a quartz seam")
 TF(STAIR,      "Is a stair")
-TF(UPSTAIR,    "Is a down staircase")
-TF(DOWNSTAIR,  "Is an up staircase")
+TF(UPSTAIR,    "Is an up staircase")
+TF(DOWNSTAIR,  "Is a down staircase")
 TF(SMOOTH,     "Should have smooth boundaries")
 TF(BRIGHT,     "Is internally lit")
+TF(FIERY,      "Is fire-based")
