@@ -70,7 +70,7 @@ typedef struct monster_lore
  */
 extern struct monster_lore *l_list;
 
-void get_attack_colors(int melee_colors[RBE_MAX], int spell_colors[RSF_MAX]);
+void get_attack_colors(int *melee_colors);
 void lore_append_kills(textblock *tb, const struct monster_race *race,
 					   const struct monster_lore *lore,
 					   const bitflag known_flags[RF_SIZE]);
@@ -99,12 +99,15 @@ void lore_append_friends(textblock *tb, const struct monster_race *race,
 						 bitflag known_flags[RF_SIZE]);
 void lore_append_spells(textblock *tb, const struct monster_race *race,
 						const struct monster_lore *lore,
-						bitflag known_flags[RF_SIZE],
-						const int spell_colors[RSF_MAX]);
+						bitflag known_flags[RF_SIZE]);
 void lore_append_attack(textblock *tb, const struct monster_race *race,
 						const struct monster_lore *lore,
-						bitflag known_flags[RF_SIZE],
-						const int melee_colors[RBE_MAX]);
+						bitflag known_flags[RF_SIZE]);
+
+void lore_learn_spell_if_has(struct monster_lore *lore, const struct monster_race *race, int flag);
+void lore_learn_spell_if_visible(struct monster_lore *lore, const struct monster *mon, int flag);
+void lore_learn_flag_if_visible(struct monster_lore *lore, const struct monster *mon, int flag);
+
 void lore_update(const struct monster_race *race, struct monster_lore *lore);
 void cheat_monster_lore(const struct monster_race *race,
 						struct monster_lore *lore);
