@@ -21,19 +21,22 @@
 
 #include "monster.h"
 
-/** Functions **/
+const char *describe_race_flag(int flag);
+void create_mon_flag_mask(bitflag *f, ...);
 struct monster_race *lookup_monster(const char *name);
 struct monster_base *lookup_monster_base(const char *name);
-bool monster_is_nonliving(struct monster_race *race);
-bool monster_is_unusual(struct monster_race *race);
 bool match_monster_bases(const struct monster_base *base, ...);
 void update_mon(struct monster *mon, struct chunk *c, bool full);
 void update_monsters(bool full);
 bool monster_carry(struct chunk *c, struct monster *mon, struct object *obj);
 void monster_swap(int y1, int x1, int y2, int x2);
 void become_aware(struct monster *m);
-bool is_mimicking(struct monster *m);
-void update_smart_learn(struct monster *m, struct player *p, int flag,
+void update_smart_learn(struct monster *mon, struct player *p, int flag,
 						int pflag, int element);
+bool find_any_nearby_injured_kin(struct chunk *c, const struct monster *mon);
+struct monster *choose_nearby_injured_kin(struct chunk *c, const struct monster *mon);
+void monster_death(struct monster *mon, bool stats);
+bool mon_take_hit(struct monster *mon, int dam, bool *fear, const char *note);
+void monster_take_terrain_damage(struct monster *mon);
 
 #endif /* MONSTER_UTILITIES_H */
