@@ -21,9 +21,11 @@
 #include "cave.h"
 #include "effects.h"
 #include "init.h"
+#include "obj-curse.h"
 #include "obj-gear.h"
-#include "obj-pile.h"
+#include "obj-knowledge.h"
 #include "obj-make.h"
+#include "obj-pile.h"
 #include "obj-power.h"
 #include "obj-slays.h"
 #include "obj-tval.h"
@@ -47,16 +49,12 @@ struct money {
 static struct money *money_type;
 static int num_money_types;
 
-static void init_obj_make(void) {
-	int i, item, lev;
+/*
+ * Initialize object allocation info
+ */
+static void alloc_init_objects(void) {
+	int item, lev;
 	int k_max = z_info->k_max;
-	struct alloc_entry *table;
-	struct ego_item *ego;
-	s16b *num;
-	s16b *aux;
-	int *money_svals;
-
-	/*** Initialize object allocation info ***/
 
 	/* Allocate and wipe */
 	obj_alloc = mem_zalloc((z_info->max_obj_depth + 1) * k_max * sizeof(byte));
@@ -65,7 +63,7 @@ static void init_obj_make(void) {
 	obj_total_great = mem_zalloc((z_info->max_obj_depth + 1) * sizeof(u32b));
 
 	/* Init allocation data */
-	for (item = 1; item < k_max; item++) {
+	for (item = 0; item < k_max; item++) {
 		const struct object_kind *kind = &k_info[item];
 
 		int min = kind->alloc_min;
@@ -89,24 +87,30 @@ static void init_obj_make(void) {
 			obj_alloc_great[(lev * k_max) + item] = rarity;
 		}
 	}
+}
 
-	/*** Initialize ego-item allocation info ***/
+/*
+ * Initialize ego-item allocation info
+ *
+ * The ego allocation probabilities table (alloc_ego_table) is sorted in
+ * order of minimum depth.  Precisely why, I'm not sure!  But that is what
+ * the code below is doing with the arrays 'num' and 'level_total'. -AS
+ */
+static void alloc_init_egos(void) {
+	int *num = mem_zalloc((z_info->max_obj_depth + 1) * sizeof(int));
+	int *level_total = mem_zalloc((z_info->max_obj_depth + 1) * sizeof(int));
 
-	num = mem_zalloc((z_info->max_obj_depth + 1) * sizeof(s16b));
-	aux = mem_zalloc((z_info->max_obj_depth + 1) * sizeof(s16b));
+	int i;
 
-	/* Scan the ego items */
-	for (i = 1; i < z_info->e_max; i++) {
-		/* Get the i'th ego item */
-		ego = &e_info[i];
+	for (i = 0; i < z_info->e_max; i++) {
+		struct ego_item *ego = &e_info[i];
 
-		/* Legal items */
-		if (ego->rarity) {
+		if (ego->alloc_prob) {
 			/* Count the entries */
 			alloc_ego_size++;
 
 			/* Group by level */
-			num[ego->level]++;
+			num[ego->alloc_min]++;
 		}
 	}
 
@@ -117,45 +121,43 @@ static void init_obj_make(void) {
 	/* Allocate the alloc_ego_table */
 	alloc_ego_table = mem_zalloc(alloc_ego_size * sizeof(alloc_entry));
 
-	/* Get the table entry */
-	table = alloc_ego_table;
-
 	/* Scan the ego-items */
-	for (i = 1; i < z_info->e_max; i++) {
-		/* Get the i'th ego item */
-		ego = &e_info[i];
+	for (i = 0; i < z_info->e_max; i++) {
+		struct ego_item *ego = &e_info[i];
 
 		/* Count valid pairs */
-		if (ego->rarity) {
-			int p, x, y, z;
-
-			/* Extract the base level */
-			x = ego->level;
-
-			/* Extract the base probability */
-			p = (100 / ego->rarity);
+		if (ego->alloc_prob) {
+			int min_level = ego->alloc_min;
 
 			/* Skip entries preceding our locale */
-			y = (x > 0) ? num[x - 1] : 0;
+			int y = (min_level > 0) ? num[min_level - 1] : 0;
 
 			/* Skip previous entries at this locale */
-			z = y + aux[x];
+			int z = y + level_total[min_level];
 
 			/* Load the entry */
-			table[z].index = i;
-			table[z].level = x;
-			table[z].prob1 = p;
-			table[z].prob2 = p;
-			table[z].prob3 = p;
+			alloc_ego_table[z].index = i;
+			alloc_ego_table[z].level = min_level;			/* Unused */
+			alloc_ego_table[z].prob1 = ego->alloc_prob;
+			alloc_ego_table[z].prob2 = ego->alloc_prob;
+			alloc_ego_table[z].prob3 = ego->alloc_prob;
 
 			/* Another entry complete for this locale */
-			aux[x]++;
+			level_total[min_level]++;
 		}
 	}
-	mem_free(aux);
-	mem_free(num);
 
-	/*** Initialize money info ***/
+	mem_free(level_total);
+	mem_free(num);
+}
+
+/*
+ * Initialize money info
+ */
+static void init_money_svals(void)
+{
+	int *money_svals;
+	int i;
 
 	/* Count the money types and make a list */
 	num_money_types = tval_sval_count("gold");
@@ -169,9 +171,16 @@ static void init_obj_make(void) {
 		money_type[i].name = string_make(kind->name);
 		money_type[i].type = money_svals[i];
 	}
+
 	mem_free(money_svals);
 }
 
+static void init_obj_make(void) {
+	alloc_init_objects();
+	alloc_init_egos();
+	init_money_svals();
+}
+
 static void cleanup_obj_make(void) {
 	int i;
 	for (i = 0; i < num_money_types; i++) {
@@ -211,6 +220,35 @@ static int get_new_attr(bitflag flags[OF_SIZE], bitflag newf[OF_SIZE])
 	return flag;
 }
 
+/**
+ * Get a random new base resist on an item
+ */
+static int random_base_resist(struct object *obj, int *resist)
+{
+	int i, r, count = 0;
+
+	/* Count the available base resists */
+	for (i = ELEM_BASE_MIN; i < ELEM_HIGH_MIN; i++)
+		if (obj->el_info[i].res_level == 0) count++;
+
+	if (count == 0) return false;
+
+	/* Pick one */
+	r = randint0(count);
+
+	/* Find the one we picked */
+	for (i = ELEM_BASE_MIN; i < ELEM_HIGH_MIN; i++) {
+		if (obj->el_info[i].res_level != 0) continue;
+		if (r == 0) {
+			*resist = i;
+			return true;
+		}
+		r--;
+	}
+
+	return false;
+}
+
 /**
  * Get a random new high resist on an item
  */
@@ -219,25 +257,25 @@ static int random_high_resist(struct object *obj, int *resist)
 	int i, r, count = 0;
 
 	/* Count the available high resists */
-	for (i = ELEM_HIGH_MIN; i <= ELEM_HIGH_MAX; i++)
+	for (i = ELEM_HIGH_MIN; i < ELEM_HIGH_MAX; i++)
 		if (obj->el_info[i].res_level == 0) count++;
 
-	if (count == 0) return FALSE;
+	if (count == 0) return false;
 
 	/* Pick one */
 	r = randint0(count);
 
 	/* Find the one we picked */
-	for (i = ELEM_HIGH_MIN; i <= ELEM_HIGH_MAX; i++) {
+	for (i = ELEM_HIGH_MIN; i < ELEM_HIGH_MAX; i++) {
 		if (obj->el_info[i].res_level != 0) continue;
 		if (r == 0) {
 			*resist = i;
-			return TRUE;
+			return true;
 		}
 		r--;
 	}
 
-	return FALSE;
+	return false;
 }
 
 
@@ -246,57 +284,46 @@ static int random_high_resist(struct object *obj, int *resist)
  */
 static struct ego_item *ego_find_random(struct object *obj, int level)
 {
-	int i, ood_chance;
+	int i;
 	long total = 0L;
 
 	alloc_entry *table = alloc_ego_table;
-	struct ego_item *ego;
-	struct ego_poss_item *poss;
 
 	/* Go through all possible ego items and find ones which fit this item */
 	for (i = 0; i < alloc_ego_size; i++) {
+		struct ego_item *ego = &e_info[table[i].index];
+
 		/* Reset any previous probability of this type being picked */
 		table[i].prob3 = 0;
 
-		if (level < table[i].level)
-			continue;
+		if (level <= ego->alloc_max) {
+			int ood_chance = MAX(2, (ego->alloc_min - level) / 3);
+			if (level >= ego->alloc_min || one_in_(ood_chance)) {
+				struct poss_item *poss;
 
-		/* Access the ego item */
-		ego = &e_info[table[i].index];
-        
-        /* enforce maximum */
-        if (level > ego->alloc_max) continue;
-        
-        /* roll for Out of Depth (ood) */
-        if (level < ego->alloc_min){
-            ood_chance = MAX(2, (ego->alloc_min - level) / 3);
-            if (!one_in_(ood_chance)) continue;
-        }
+				for (poss = ego->poss_items; poss; poss = poss->next)
+					if (poss->kidx == obj->kind->kidx) {
+						table[i].prob3 = table[i].prob2;
+						break;
+					}
 
-		/* XXX Ignore cursed items for now */
-		if (cursed_p(ego->flags)) continue;
-
-		for (poss = ego->poss_items; poss; poss = poss->next)
-			if (poss->kidx == obj->kind->kidx) {
-				table[i].prob3 = table[i].prob2;
-				break;
+				/* Total */
+				total += table[i].prob3;
 			}
-
-		/* Total */
-		total += table[i].prob3;
+		}
 	}
 
 	if (total) {
 		long value = randint0(total);
 		for (i = 0; i < alloc_ego_size; i++) {
 			/* Found the entry */
-			if (value < table[i].prob3) break;
-
-			/* Decrement */
-			value = value - table[i].prob3;
+			if (value < table[i].prob3) {
+				return &e_info[table[i].index];
+			} else {
+				/* Decrement */
+				value = value - table[i].prob3;
+			}
 		}
-
-		return &e_info[table[i].index];
 	}
 
 	return NULL;
@@ -308,24 +335,33 @@ static struct ego_item *ego_find_random(struct object *obj, int level)
  */
 void ego_apply_magic(struct object *obj, int level)
 {
-	int i, x, resist = 0;
+	int i, x, resist = 0, pick = 0;
 	bitflag newf[OF_SIZE];
 
+	/* Resist or power? */
+	if (kf_has(obj->ego->kind_flags, KF_RAND_RES_POWER))
+		pick = randint1(3);
+
 	/* Extra powers */
 	if (kf_has(obj->ego->kind_flags, KF_RAND_SUSTAIN)) {
-		create_mask(newf, FALSE, OFT_SUST, OFT_MAX);
+		create_obj_flag_mask(newf, false, OFT_SUST, OFT_MAX);
 		of_on(obj->flags, get_new_attr(obj->flags, newf));
-	}
-	else if (kf_has(obj->ego->kind_flags, KF_RAND_POWER)) {
-		create_mask(newf, FALSE, OFT_PROT, OFT_MISC, OFT_MAX);
+	} else if (kf_has(obj->ego->kind_flags, KF_RAND_POWER) || (pick == 1)) {
+		create_obj_flag_mask(newf, false, OFT_PROT, OFT_MISC, OFT_MAX);
 		of_on(obj->flags, get_new_attr(obj->flags, newf));
-	}
-	else if (kf_has(obj->ego->kind_flags, KF_RAND_HI_RES))
+	} else if (kf_has(obj->ego->kind_flags, KF_RAND_BASE_RES) || (pick > 1)) {
+		/* Get a base resist if available, mark it as random */
+		if (random_base_resist(obj, &resist)) {
+			obj->el_info[resist].res_level = 1;
+			obj->el_info[resist].flags |= EL_INFO_RANDOM;
+		}
+	} else if (kf_has(obj->ego->kind_flags, KF_RAND_HI_RES)) {
 		/* Get a high resist if available, mark it as random */
 		if (random_high_resist(obj, &resist)) {
 			obj->el_info[resist].res_level = 1;
 			obj->el_info[resist].flags |= EL_INFO_RANDOM;
 		}
+	}
 
 	/* Apply extra obj->ego bonuses */
 	obj->to_h += randcalc(obj->ego->to_h, level, RANDOMISE);
@@ -342,9 +378,10 @@ void ego_apply_magic(struct object *obj, int level)
 	of_union(obj->flags, obj->ego->flags);
 	of_diff(obj->flags, obj->ego->flags_off);
 
-	/* Add slays and brands */
-	copy_slay(&obj->slays, obj->ego->slays);
-	copy_brand(&obj->brands, obj->ego->brands);
+	/* Add slays, brands and curses */
+	copy_slays(&obj->slays, obj->ego->slays);
+	copy_brands(&obj->brands, obj->ego->brands);
+	copy_curses(obj, obj->ego->curses);
 
 	/* Add resists */
 	for (i = 0; i < ELEM_MAX; i++) {
@@ -416,13 +453,6 @@ static void make_ego_item(struct object *obj, int level)
 	if (obj->ego)
 		ego_apply_magic(obj, level);
 
-	/* Ego lights are always known as such (why? - NRM) */
-	/* Might as well make them known not artifacts as well - NRM */
-	if (tval_is_light(obj)) {
-		id_on(obj->id_flags, ID_EGO_ITEM);
-		id_on(obj->id_flags, ID_ARTIFACT);
-	}
-
 	return;
 }
 
@@ -430,12 +460,12 @@ static void make_ego_item(struct object *obj, int level)
 /*** Make an artifact ***/
 
 /**
- * Copy artifact data to a normal object, and set various slightly hacky
- * globals.
+ * Copy artifact data to a normal object.
  */
 void copy_artifact_data(struct object *obj, const struct artifact *art)
 {
 	int i;
+	struct object_kind *kind = lookup_kind(art->tval, art->sval);
 
 	/* Extract the data */
 	for (i = 0; i < OBJ_MOD_MAX; i++)
@@ -447,12 +477,27 @@ void copy_artifact_data(struct object *obj, const struct artifact *art)
 	obj->to_h = art->to_h;
 	obj->to_d = art->to_d;
 	obj->weight = art->weight;
-	obj->activation = art->activation;
-	if (art->time.base != 0)
+
+	/* Activations can come from the artifact or the kind */
+	if (art->activation) {
+		obj->activation = art->activation;
 		obj->time = art->time;
+	} else if (kind->activation) {
+		obj->activation = kind->activation;
+		obj->time = kind->time;
+	}
+
+	/* Fix for artifact lights */
+	of_off(obj->flags, OF_TAKES_FUEL);
+	of_off(obj->flags, OF_BURNS_OUT);
+
+	/* Timeouts are always 0 */
+	obj->timeout = 0;
+
 	of_union(obj->flags, art->flags);
-	copy_slay(&obj->slays, art->slays);
-	copy_brand(&obj->brands, art->brands);
+	copy_slays(&obj->slays, art->slays);
+	copy_brands(&obj->brands, art->brands);
+	copy_curses(obj, art->curses);
 	for (i = 0; i < ELEM_MAX; i++) {
 		/* Take the larger of artifact and base object resist levels */
 		obj->el_info[i].res_level =
@@ -480,7 +525,7 @@ static struct object *make_artifact_special(int level)
 	struct object *new_obj;
 
 	/* No artifacts, do nothing */
-	if (OPT(birth_no_artifacts))
+	if (OPT(player, birth_no_artifacts))
 		return NULL;
 
 	/* No artifacts in the town */
@@ -539,7 +584,7 @@ static struct object *make_artifact_special(int level)
 		copy_artifact_data(new_obj, art);
 
 		/* Mark the artifact as "created" */
-		art->created = TRUE;
+		art->created = true;
 
 		/* Success */
 		return new_obj;
@@ -562,22 +607,18 @@ static struct object *make_artifact_special(int level)
 static bool make_artifact(struct object *obj)
 {
 	int i;
-	bool art_ok = TRUE;
+	bool art_ok = true;
 
 	/* Make sure birth no artifacts isn't set */
-	if (OPT(birth_no_artifacts)) art_ok = FALSE;
+	if (OPT(player, birth_no_artifacts)) art_ok = false;
 
-	/* Special handling of quest artifacts */
-	if (kf_has(obj->kind->kind_flags, KF_QUEST_ART))
-		art_ok = TRUE;
-
-	if (!art_ok) return (FALSE);
+	if (!art_ok) return (false);
 
 	/* No artifacts in the town */
-	if (!player->depth) return (FALSE);
+	if (!player->depth) return (false);
 
 	/* Paranoia -- no "plural" artifacts */
-	if (obj->number != 1) return (FALSE);
+	if (obj->number != 1) return (false);
 
 	/* Check the artifact list (skip the "specials") */
 	for (i = 0; !obj->artifact && i < z_info->a_max; i++) {
@@ -622,11 +663,11 @@ static bool make_artifact(struct object *obj)
 
 	if (obj->artifact) {
 		copy_artifact_data(obj, obj->artifact);
-		obj->artifact->created = TRUE;
-		return TRUE;
+		obj->artifact->created = true;
+		return true;
 	}
 
-	return FALSE;
+	return false;
 }
 
 
@@ -639,28 +680,23 @@ static bool make_artifact(struct object *obj)
  * Since this is now in no way marked as fake, we must make sure this function
  * is never used to create an actual game object
  */
-bool make_fake_artifact(struct object *obj, struct artifact *artifact)
+bool make_fake_artifact(struct object *obj, const struct artifact *artifact)
 {
 	struct object_kind *kind;
 
 	/* Don't bother with empty artifacts */
-	if (!artifact->tval) return FALSE;
+	if (!artifact->tval) return false;
 
 	/* Get the "kind" index */
 	kind = lookup_kind(artifact->tval, artifact->sval);
-	if (!kind) return FALSE;
+	if (!kind) return false;
 
 	/* Create the artifact */
 	object_prep(obj, kind, 0, MAXIMISE);
-
-	/* Save the name */
-	obj->artifact = artifact;
-
-	/* Extract the fields */
+	obj->artifact = (struct artifact *)artifact;
 	copy_artifact_data(obj, artifact);
 
-	/* Success */
-	return (TRUE);
+	return (true);
 }
 
 
@@ -708,28 +744,6 @@ static void apply_magic_armour(struct object *obj, int level, int power)
 }
 
 
-/**
- * Set some ID flags for items where the flavour is already known
- */
-void id_set_aware(struct object *obj)
-{
-	int i;
-
-	/* Know pval and effect */
-	id_on(obj->id_flags, ID_PVAL);
-	id_on(obj->id_flags, ID_EFFECT);
-
-	/* Jewelry with fixed bonuses gets more info now */
-	if (tval_is_jewelry(obj)) {
-		if (!randcalc_varies(obj->kind->to_h)) id_on(obj->id_flags, ID_TO_H);
-		if (!randcalc_varies(obj->kind->to_d)) id_on(obj->id_flags, ID_TO_D);
-		if (!randcalc_varies(obj->kind->to_a)) id_on(obj->id_flags, ID_TO_A);
-		for (i = 0; i < OBJ_MOD_MAX; i++)
-			if (!randcalc_varies(obj->kind->modifiers[i]))
-				id_on(obj->id_flags, ID_MOD_MIN + i);
-	}
-}
-
 /**
  * Wipe an object clean and make it a standard object of the specified kind.
  */
@@ -786,33 +800,54 @@ void object_prep(struct object *obj, struct object_kind *k, int lev,
 	obj->to_d = randcalc(k->to_d, lev, rand_aspect);
 	obj->to_a = randcalc(k->to_a, lev, rand_aspect);
 
-	/* Default slays and brands */
-	copy_slay(&obj->slays, k->slays);
-	copy_brand(&obj->brands, k->brands);
+	/* Default slays, brands and curses */
+	copy_slays(&obj->slays, k->slays);
+	copy_brands(&obj->brands, k->brands);
+	copy_curses(obj, k->curses);
 
 	/* Default resists */
 	for (i = 0; i < ELEM_MAX; i++) {
 		obj->el_info[i].res_level = k->el_info[i].res_level;
 		obj->el_info[i].flags = k->el_info[i].flags;
 		obj->el_info[i].flags |= k->base->el_info[i].flags;
-
-		/* Unresistables have no hidden properties */
-		if (i > ELEM_HIGH_MAX)
-			obj->el_info[i].flags |= EL_INFO_KNOWN;
 	}
-
-	/* Extra info for flavour-aware items */
-	if (obj->kind->flavor && obj->kind->aware)
-		id_set_aware(obj);
 }
 
+/**
+ * Attempt to apply curses to an object, with a corresponding increase in
+ * generation level of the object
+ */
+static int apply_curse(struct object *obj, int lev)
+{
+	int pick, max_curses = randint1(4);
+	int power = randint1(9) + 10 * m_bonus(9, lev);
+	int new_lev = lev;
+
+	if (of_has(obj->flags, OF_BLESSED)) return lev;
+
+	while (max_curses--) {
+		/* Try to curse it */
+		int tries = 3;
+		while (tries--) {
+			pick = randint1(z_info->curse_max - 1);
+			if (curses[pick].poss[obj->tval]) {
+				if (append_object_curse(obj, pick, power)) {
+					new_lev += randint1(1 + power / 10);
+				}
+				break;
+			}
+		}
+	}
+
+	return new_lev;
+}
 
 /**
  * Applying magic to an object, which includes creating ego-items, and applying
  * random bonuses,
  *
  * The `good` argument forces the item to be at least `good`, and the `great`
- * argument does likewise.  Setting `allow_artifacts` to TRUE allows artifacts
+ * argument does likewise.  Setting `allow_artifacts` to true allows artifacts
  * to be created here.
  *
  * If `good` or `great` are not set, then the `lev` argument controls the
@@ -830,14 +865,14 @@ int apply_magic(struct object *obj, int lev, bool allow_artifacts, bool good,
 	/* Chance of being `good` and `great` */
 	/* This has changed over the years:
 	 * 3.0.0:   good = MIN(75, lev + 10);      great = MIN(20, lev / 2); 
-	 * 3.3.0:	good = (lev + 2) * 3;          great = MIN(lev / 4 + lev, 50);
-     * 3.4.0:   good = (2 * lev) + 5
-     * 3.4 was in between 3.0 and 3.3, 3.5 attempts to keep the same
-     * area under the curve as 3.4, but make the generation chances
-     * flatter.  This depresses good items overall since more items
-     * are created deeper. 
-     * This change is meant to go in conjunction with the changes
-     * to ego item allocation levels. (-fizzix)
+	 * 3.3.0:   good = (lev + 2) * 3;          great = MIN(lev / 4 + lev, 50);
+	 * 3.4.0:   good = (2 * lev) + 5
+	 * 3.4 was in between 3.0 and 3.3, 3.5 attempts to keep the same
+	 * area under the curve as 3.4, but make the generation chances
+	 * flatter.  This depresses good items overall since more items
+	 * are created deeper. 
+	 * This change is meant to go in conjunction with the changes
+	 * to ego item allocation levels. (-fizzix)
 	 */
 	int good_chance = (33 + lev);
 	int great_chance = 30;
@@ -860,9 +895,9 @@ int apply_magic(struct object *obj, int lev, bool allow_artifacts, bool good,
 
 		/* Get two rolls if forced great */
 		if (great) rolls = 2;
-        
-        /* Give some extra rolls for uniques and acq scrolls */
-        if (extra_roll) rolls += 2;
+		
+		/* Give some extra rolls for uniques and acq scrolls */
+		if (extra_roll) rolls += 2;
 
 		/* Roll for artifacts if allowed */
 		for (i = 0; i < rolls; i++)
@@ -873,21 +908,23 @@ int apply_magic(struct object *obj, int lev, bool allow_artifacts, bool good,
 	if (power == 2)
 		make_ego_item(obj, lev);
 
+	/* Give it a chance to be cursed */
+	if (one_in_(20) && tval_is_wearable(obj)) {
+		lev = apply_curse(obj, lev);
+	}
+
 	/* Apply magic */
 	if (tval_is_weapon(obj)) {
 		apply_magic_weapon(obj, lev, power);
-	}
-	else if (tval_is_armor(obj)) {
+	} else if (tval_is_armor(obj)) {
 		apply_magic_armour(obj, lev, power);
-	}
-	else if (tval_is_ring(obj)) {
+	} else if (tval_is_ring(obj)) {
 		if (obj->sval == lookup_sval(obj->tval, "Speed")) {
 			/* Super-charge the ring */
 			while (one_in_(2))
 				obj->modifiers[OBJ_MOD_SPEED]++;
 		}
-	}
-	else if (tval_is_chest(obj)) {
+	} else if (tval_is_chest(obj)) {
 		/* Hack -- skip ruined chests */
 		if (obj->kind->level > 0) {
 			/* Hack -- pick a "difficulty" */
@@ -932,8 +969,8 @@ bool kind_is_good(const struct object_kind *kind)
 		case TV_HELM:
 		case TV_CROWN:
 		{
-			if (randcalc(kind->to_a, 0, MINIMISE) < 0) return (FALSE);
-			return TRUE;
+			if (randcalc(kind->to_a, 0, MINIMISE) < 0) return (false);
+			return true;
 		}
 
 		/* Weapons -- Good unless damaged */
@@ -943,25 +980,25 @@ bool kind_is_good(const struct object_kind *kind)
 		case TV_POLEARM:
 		case TV_DIGGING:
 		{
-			if (randcalc(kind->to_h, 0, MINIMISE) < 0) return (FALSE);
-			if (randcalc(kind->to_d, 0, MINIMISE) < 0) return (FALSE);
-			return TRUE;
+			if (randcalc(kind->to_h, 0, MINIMISE) < 0) return (false);
+			if (randcalc(kind->to_d, 0, MINIMISE) < 0) return (false);
+			return true;
 		}
 
 		/* Ammo -- Arrows/Bolts are good */
 		case TV_BOLT:
 		case TV_ARROW:
 		{
-			return TRUE;
+			return true;
 		}
 	}
 
 	/* Anything with the GOOD flag */
 	if (kf_has(kind->kind_flags, KF_GOOD))
-		return TRUE;
+		return true;
 
 	/* Assume not good */
-	return (FALSE);
+	return (false);
 }
 
 
@@ -980,7 +1017,7 @@ static struct object_kind *get_obj_num_by_kind(int level, bool good, int tval)
 	ind = level * z_info->k_max;
 
 	/* Get new total */
-	for (item = 1; item < z_info->k_max; item++)
+	for (item = 0; item < z_info->k_max; item++)
 		if (objkind_byid(item)->tval == tval)
 			total += objects[ind + item];
 
@@ -989,7 +1026,7 @@ static struct object_kind *get_obj_num_by_kind(int level, bool good, int tval)
 	
 	value = randint0(total);
 	
-	for (item = 1; item < z_info->k_max; item++)
+	for (item = 0; item < z_info->k_max; item++)
 		if (objkind_byid(item)->tval == tval) {
 			if (value < objects[ind + item]) break;
 
@@ -1028,7 +1065,7 @@ struct object_kind *get_obj_num(int level, bool good, int tval)
 	
 	if (!good) {
 		value = randint0(obj_total[level]);
-		for (item = 1; item < z_info->k_max; item++) {
+		for (item = 0; item < z_info->k_max; item++) {
 			/* Found it */
 			if (value < obj_alloc[ind + item]) break;
 
@@ -1037,7 +1074,7 @@ struct object_kind *get_obj_num(int level, bool good, int tval)
 		}
 	} else {
 		value = randint0(obj_total_great[level]);
-		for (item = 1; item < z_info->k_max; item++) {
+		for (item = 0; item < z_info->k_max; item++) {
 			/* Found it */
 			if (value < obj_alloc_great[ind + item]) break;
 
@@ -1075,12 +1112,12 @@ struct object *make_object(struct chunk *c, int lev, bool good, bool great,
 	if (one_in_(good ? 10 : 1000)) {
 		new_obj = make_artifact_special(lev);
 		if (new_obj) {
-			if (value) *value = object_value_real(new_obj, 1, FALSE, TRUE);
+			if (value) *value = object_value_real(new_obj, 1);
 			return new_obj;
 		}
 
 		/* If we failed to make an artifact, the player gets a good item */
-		good = TRUE;
+		good = true;
 	}
 
 	/* Base level for the object */
@@ -1094,21 +1131,21 @@ struct object *make_object(struct chunk *c, int lev, bool good, bool great,
 	/* Make the object, prep it and apply magic */
 	new_obj = object_new();
 	object_prep(new_obj, kind, lev, RANDOMISE);
-	apply_magic(new_obj, lev, TRUE, good, great, extra_roll);
+	apply_magic(new_obj, lev, true, good, great, extra_roll);
 
 	/* Generate multiple items */
 	if (kind->gen_mult_prob >= randint1(100))
 		new_obj->number = randcalc(kind->stack_size, lev, RANDOMISE);
 
-	if (new_obj->number > z_info->stack_size)
-		new_obj->number = z_info->stack_size;
+	if (new_obj->number > new_obj->kind->base->max_stack)
+		new_obj->number = new_obj->kind->base->max_stack;
 
 	/* Get the value */
 	if (value)
-		*value = object_value_real(new_obj, new_obj->number, FALSE, TRUE);
+		*value = object_value_real(new_obj, new_obj->number);
 
 	/* Boost of 20% per level OOD for uncursed objects */
-	if (!cursed_p(new_obj->flags) && (kind->alloc_min > c->depth)) {
+	if ((!new_obj->curses) && (kind->alloc_min > c->depth)) {
 		if (value) *value += (kind->alloc_min - c->depth) * (*value / 5);
 	}
 
@@ -1126,14 +1163,14 @@ void acquirement(int y1, int x1, int level, int num, bool great)
 	/* Acquirement */
 	while (num--) {
 		/* Make a good (or great) object (if possible) */
-		nice_obj = make_object(cave, level, TRUE, great, TRUE, NULL, 0);
+		nice_obj = make_object(cave, level, true, great, true, NULL, 0);
 		if (!nice_obj) continue;
 
 		nice_obj->origin = ORIGIN_ACQUIRE;
 		nice_obj->origin_depth = player->depth;
 
 		/* Drop the object */
-		drop_near(cave, nice_obj, 0, y1, x1, TRUE);
+		drop_near(cave, &nice_obj, 0, y1, x1, true);
 	}
 }
 
@@ -1175,26 +1212,27 @@ struct object_kind *money_kind(const char *name, int value)
  */
 struct object *make_gold(int lev, char *coin_type)
 {
-	/* This average is 20 at dlev0, 100 at dlev40, 220 at dlev100. */
-	s32b avg = (18 * lev)/10 + 18;
-	s32b spread = lev + 10;
-	s32b value = rand_spread(avg, spread);
+	/* This average is 16 at dlev0, 80 at dlev40, 176 at dlev100. */
+	int avg = (16 * lev)/10 + 16;
+	int spread = lev + 10;
+	int value = rand_spread(avg, spread);
 	struct object *new_gold = mem_zalloc(sizeof(*new_gold)); 
 
 	/* Increase the range to infinite, moving the average to 110% */
-	while (one_in_(100) && value * 10 <= MAX_SHORT)
+	while (one_in_(100) && value * 10 <= SHRT_MAX)
 		value *= 10;
 
 	/* Prepare a gold object */
 	object_prep(new_gold, money_kind(coin_type, value), lev, RANDOMISE);
 
 	/* If we're playing with no_selling, increase the value */
-	if (OPT(birth_no_selling) && player->depth)
-		value = value * MIN(5, player->depth);
+	if (OPT(player, birth_no_selling) && player->depth)
+		value *= MIN(5, player->depth);
 
 	/* Cap gold at max short (or alternatively make pvals s32b) */
-	if (value > MAX_SHORT)
-		value = MAX_SHORT;
+	if (value >= SHRT_MAX) {
+		value = SHRT_MAX - randint0(200);
+	}
 
 	new_gold->pval = value;
 
