@@ -22,9 +22,9 @@
 void do_cmd_redraw(void);
 void do_cmd_xxx_options(void);
 void do_cmd_unknown(void);
+void do_cmd_version(void);
 void textui_cmd_suicide(void);
 void textui_cmd_debug(void);
-void do_cmd_load_screen(void);
 void html_screenshot(const char *path, int mode);
 void do_cmd_save_screen(void);
 void textui_cmd_rest(void);
