@@ -23,7 +23,7 @@
 #include "cmd-core.h"
 #include "game-event.h"
 
-bool arg_wizard;
+extern bool arg_wizard;
 extern char savefile[1024];
 
 void cmd_init(void);
@@ -35,7 +35,7 @@ errr textui_get_cmd(cmd_context context);
 void check_for_player_interrupt(game_event_type type, game_event_data *data,
 								void *user);
 void play_game(bool new_game);
-void savefile_set_name(const char *fname);
+void savefile_set_name(const char *fname, bool make_safe, bool strip_suffix);
 void save_game(void);
 void close_game(void);
 
