@@ -1013,8 +1013,8 @@ byte tile_height = 1;           /* Tile height in units of font height */
 /**
  * Helper variables for large cursor
  */
-bool bigcurs = FALSE;
-bool smlcurs = TRUE;
+bool bigcurs = false;
+bool smlcurs = true;
 
 
 /**
@@ -1176,7 +1176,7 @@ errr Term_fresh(void)
 		Term_xtra(TERM_XTRA_CLEAR, 0);
 
 		/* Hack -- clear all "cursor" data */
-		old->cv = old->cu = FALSE;
+		old->cv = old->cu = false;
 		old->cx = old->cy = 0;
 
 		/* Wipe each row */
@@ -1208,7 +1208,7 @@ errr Term_fresh(void)
 		}
 
 		/* Forget "total erase" */
-		Term->total_erase = FALSE;
+		Term->total_erase = false;
 	}
 
 
@@ -1710,7 +1710,7 @@ errr Term_clear(void)
 	Term->y2 = h - 1;
 
 	/* Force "total erase" */
-	Term->total_erase = TRUE;
+	Term->total_erase = true;
 
 	/* Success */
 	return (0);
@@ -1726,7 +1726,7 @@ errr Term_clear(void)
 errr Term_redraw(void)
 {
 	/* Force "total erase" */
-	Term->total_erase = TRUE;
+	Term->total_erase = true;
 
 	/* Hack -- Refresh */
 	Term_fresh();
@@ -2032,12 +2032,12 @@ errr Term_inkey(ui_event *ch, bool wait, bool take)
 		/* Process pending events while necessary */
 		while (Term->key_head == Term->key_tail)
 			/* Process events (wait for one) */
-			Term_xtra(TERM_XTRA_EVENT, TRUE);
+			Term_xtra(TERM_XTRA_EVENT, true);
 	else
 		/* Process pending events if necessary */
 		if (Term->key_head == Term->key_tail)
 			/* Process events (do not wait) */
-			Term_xtra(TERM_XTRA_EVENT, FALSE);
+			Term_xtra(TERM_XTRA_EVENT, false);
 
 	/* No keys are ready */
 	if (Term->key_head == Term->key_tail) return (1);
@@ -2297,7 +2297,7 @@ errr Term_resize(int w, int h)
 	Term->hgt = h;
 
 	/* Force "total erase" */
-	Term->total_erase = TRUE;
+	Term->total_erase = true;
 
 	/* Assume change */
 	for (i = 0; i < h; i++) {
@@ -2342,10 +2342,10 @@ errr Term_activate(term *t)
 		if (t->init_hook) (*t->init_hook)(t);
 
 		/* Remember */
-		t->active_flag = TRUE;
+		t->active_flag = true;
 
 		/* Assume mapped */
-		t->mapped_flag = TRUE;
+		t->mapped_flag = true;
 	}
 
 	/* Remember the Term */
@@ -2371,10 +2371,10 @@ errr term_nuke(term *t)
 		if (t->nuke_hook) (*t->nuke_hook)(t);
 
 		/* Remember */
-		t->active_flag = FALSE;
+		t->active_flag = false;
 
 		/* Assume not mapped */
-		t->mapped_flag = FALSE;
+		t->mapped_flag = false;
 	}
 
 
@@ -2476,7 +2476,7 @@ errr term_init(term *t, int w, int h, int k)
 	t->y2 = h - 1;
 
 	/* Force "total erase" */
-	t->total_erase = TRUE;
+	t->total_erase = true;
 
 	/* Default "blank" */
 	t->attr_blank = 0;
